//
//   File : kvi_menubar.cpp
//   Creation date : Wen Jun 21 2000 13:12:11 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#define __KVIRC__
#include "kvi_menubar.h"
#include "kvi_app.h"
#include "kvi_locale.h"
#include "kvi_frame.h"
#include "kvi_mdimanager.h"
#include "kvi_iconmanager.h"
#include "kvi_internalcmd.h"
#include "kvi_settings.h"
#include "kvi_ircurl.h"
#include "kvi_uparser.h"
#include "kvi_console.h"
#include "kvi_popupmenu.h"
#include "kvi_malloc.h"
#include "kvi_moduleextensiontoolbar.h"

#ifdef COMPILE_SCRIPTTOOLBAR
	#include "kvi_scripttoolbar.h"
	#include "kvi_scripttoolbar_manager.h"
	#include "kvi_scripttoolbar_definition.h"
#endif


#include <qpopupmenu.h>

KviMenuBar::KviMenuBar(KviFrame * par,const char * name)
: KviTalMenuBar(par,name)
{
	m_pFrm = par;

	m_iNumDefaultItems = 0;
	m_pDefaultItemId = 0;

	QPopupMenu * pop = new QPopupMenu(this);
	connect(pop,SIGNAL(aboutToShow()),this,SLOT(setupMainPopup()));
	addDefaultItem("&KVIrc",pop);

	m_pRecentServersPopup = new QPopupMenu(this);
	connect(m_pRecentServersPopup,SIGNAL(aboutToShow()),this,SLOT(setupRecentServersPopup()));
	connect(m_pRecentServersPopup,SIGNAL(activated(int)),this,SLOT(newConnectionToServer(int)));

	m_pScriptItemList = 0;

	QPopupMenu * opt = new QPopupMenu(this);
// FIXME: #warning "Toggle these items on the fly ?"
	int id = opt->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_OPTIONS)),__tr2qs("&Options"),m_pFrm,SLOT(executeInternalCommand(int)));
	opt->setItemParameter(id,KVI_INTERNALCOMMAND_OPTIONS_DIALOG);
	id = opt->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_REGUSERS)),__tr2qs("&Registered users"),m_pFrm,SLOT(executeInternalCommand(int)));
	opt->setItemParameter(id,KVI_INTERNALCOMMAND_REGUSER_EDIT);
//	id = opt->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SCRIPTCENTER)),__tr2qs("&Script center"),m_pFrm,SLOT(executeInternalCommand(int)));
//	opt->setItemParameter(id,KVI_INTERNALCOMMAND_SCRIPTCENTER_OPEN);
	opt->insertSeparator();
	opt->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_FLOPPY)),__tr2qs("&Save configuration"),g_pApp,SLOT(saveConfiguration()));

	addDefaultItem(__tr2qs("&Settings"),opt);


	m_pTools = new QPopupMenu(this);
	m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_ICONMANAGER)),__tr2qs("&Icon widget"),g_pIconManager,SLOT(showIconWidget()));
//	id = m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_ICQ)),__c2q(__tr("Ic&q window")),m_pFrm,SLOT(executeInternalCommand(int)));
//	m_pTools->setItemParameter(id,KVI_INTERNALCOMMAND_ICQ_OPEN);
//	id = m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_GNUTELLA)),__tr2qs("&Gnutella window"),m_pFrm,SLOT(executeInternalCommand(int)));
//	m_pTools->setItemParameter(id,KVI_INTERNALCOMMAND_GNUTELLA_OPEN);
#ifdef COMPILE_KDE_SUPPORT
	id = m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_TERMINAL)),__tr2qs("&Terminal window"),m_pFrm,SLOT(executeInternalCommand(int)));
	m_pTools->setItemParameter(id,KVI_INTERNALCOMMAND_TERM_OPEN);
#endif

//    m_pDirectoryBrowserPopup = new QPopupMenu(this);
//    id = m_pDirectoryBrowserPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_FOLDER)),__tr2qs("&Directory browser (embedded)"),m_pFrm,SLOT(executeInternalCommand(int)));
//    m_pDirectoryBrowserPopup->setItemParameter(id,KVI_INTERNALCOMMAND_BROWSER_OPEN);
//    id = m_pDirectoryBrowserPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_FOLDER)),__tr2qs("&Directory browser (MDI)"),m_pFrm,SLOT(executeInternalCommand(int)));
//    m_pDirectoryBrowserPopup->setItemParameter(id,KVI_INTERNALCOMMAND_BROWSER_OPEN_MDI);
//    m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_FOLDER)),__tr2qs("&Directory browser"),m_pDirectoryBrowserPopup);

    m_pLogViewerPopup = new QPopupMenu(this);
    id = m_pLogViewerPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_LOG)),__tr2qs("&Log viewer (toplevel)"),m_pFrm,SLOT(executeInternalCommand(int)));
    m_pLogViewerPopup->setItemParameter(id,KVI_INTERNALCOMMAND_LOGVIEWER_OPEN);
    id = m_pLogViewerPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_LOG)),__tr2qs("Log viewer (&MDI)"),m_pFrm,SLOT(executeInternalCommand(int)));
    m_pLogViewerPopup->setItemParameter(id,KVI_INTERNALCOMMAND_LOGVIEWERMDI_OPEN);
    m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_LOG)),__tr2qs("&Log viewer"),m_pLogViewerPopup);

	id = m_pTools->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_OFFER)),__tr2qs("&Offer list"),m_pFrm,SLOT(executeInternalCommand(int)));
	m_pTools->setItemParameter(id,KVI_INTERNALCOMMAND_OFFER_WINDOW);

    

	m_pTools->insertSeparator();

	m_pToolbarsPopup = new QPopupMenu(this);
	connect(m_pToolbarsPopup,SIGNAL(aboutToShow()),this,SLOT(setupToolbarsPopup()));
	connect(m_pToolbarsPopup,SIGNAL(activated(int)),this,SLOT(toolbarsPopupSelected(int)));

	m_pTools->insertItem(__tr2qs("&Toolbars"),m_pToolbarsPopup);


	addDefaultItem(__tr2qs("&Tools"),m_pTools);

	QPopupMenu * script = new QPopupMenu(this);

	id = script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_ALIAS)),__tr2qs("Aliases ..."),m_pFrm,SLOT(executeInternalCommand(int)));
	script->setItemParameter(id,KVI_INTERNALCOMMAND_ALIASEDITOR_OPEN);
	id = script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_EVENT)),__tr2qs("Events ..."),m_pFrm,SLOT(executeInternalCommand(int)));
	script->setItemParameter(id,KVI_INTERNALCOMMAND_EVENTEDITOR_OPEN);
	id = script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_POPUP)),__tr2qs("Popups ..."),m_pFrm,SLOT(executeInternalCommand(int)));
	script->setItemParameter(id,KVI_INTERNALCOMMAND_POPUPEDITOR_OPEN);
	id = script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_RAWEVENT)),__tr2qs("Raw events ..."),m_pFrm,SLOT(executeInternalCommand(int)));
	script->setItemParameter(id,KVI_INTERNALCOMMAND_RAWEDITOR_OPEN);
#ifdef COMPILE_SCRIPTTOOLBAR
	id = script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_TOOLBAR)),__tr2qs("Toolbars ..."),m_pFrm,SLOT(executeInternalCommand(int)));
	script->setItemParameter(id,KVI_INTERNALCOMMAND_TOOLBAREDITOR_OPEN);
#endif // COMPILE_SCRIPTTOOLBAR
	script->insertSeparator();
	id = script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_BOMB)),__tr2qs("New code tester"),m_pFrm,SLOT(executeInternalCommand(int)));
	script->setItemParameter(id,KVI_INTERNALCOMMAND_CODETESTER_OPEN);

	script->insertSeparator();
	script->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_TEXTEXCLAMATIVE)),__tr2qs("Restore default script"),g_pApp,SLOT(restoreDefaultScript()));

	addDefaultItem(__tr2qs("&Scripting"),script);


	addDefaultItem(__tr2qs("&Window"),par->mdiManager()->windowPopup());

	QPopupMenu * help = new QPopupMenu(this);
	id = help->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_HELP)),__tr2qs("&Help (static browser)"),m_pFrm,SLOT(executeInternalCommand(int)));
	help->setItemParameter(id,KVI_INTERNALCOMMAND_HELP_NEWSTATICWINDOW);
	id = help->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_MDIHELP)),__tr2qs("Help (&MDI browser)"),m_pFrm,SLOT(executeInternalCommand(int)));
	help->setItemParameter(id,KVI_INTERNALCOMMAND_HELP_NEWMDIWINDOW);
	help->insertSeparator();
	id = help->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_IDEA)),__tr2qs("&Did you know..."),m_pFrm,SLOT(executeInternalCommand(int)));
	help->setItemParameter(id,KVI_INTERNALCOMMAND_TIP_OPEN);
	help->insertSeparator();
	id = help->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_KVIRC)),__tr2qs("About &KVIrc..."),m_pFrm,SLOT(executeInternalCommand(int)));
	help->setItemParameter(id,KVI_INTERNALCOMMAND_ABOUT_ABOUTKVIRC);
	help->insertSeparator();
	id = help->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_HOMEPAGE)),__tr2qs("KVIrc home&page"),m_pFrm,SLOT(executeInternalCommand(int)));
	help->setItemParameter(id,KVI_INTERNALCOMMAND_KVIRC_HOMEPAGE);

	addDefaultItem(__tr2qs("&Help"),help);
}

KviMenuBar::~KviMenuBar()
{
	if(m_pScriptItemList)delete m_pScriptItemList;
	if(m_pDefaultItemId)kvi_free(m_pDefaultItemId);
    if(m_pLogViewerPopup)delete m_pLogViewerPopup;
}

void KviMenuBar::addDefaultItem(const QString &text,QPopupMenu * pop)
{
	m_iNumDefaultItems++;
	m_pDefaultItemId = (int *)kvi_realloc((void *)m_pDefaultItemId,sizeof(int) * m_iNumDefaultItems);
	m_pDefaultItemId[m_iNumDefaultItems - 1] = insertItem(text,pop);
}

void KviMenuBar::setupMainPopup()
{
	QPopupMenu * main = (QPopupMenu *)sender();

	main->clear();

	main->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CONSOLE)),__tr2qs("&New irc context"),m_pFrm,SLOT(newConsole()));
	main->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WORLD)),__tr2qs("&New connection to"),m_pRecentServersPopup);

	main->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_RAW)),__tr2qs("New &frame window"),g_pApp,SLOT(createNewFrame()));

	main->insertSeparator();

	if(m_pFrm->dockExtension())
	{
		int id = main->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_DOCKWIDGET)),__tr2qs("Hide &dock widget"),m_pFrm,SLOT(executeInternalCommand(int)));
		main->setItemParameter(id,KVI_INTERNALCOMMAND_DOCKWIDGET_HIDE);
	} else {
		int id = main->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_DOCKWIDGET)),__tr2qs("Show &dock widget"),m_pFrm,SLOT(executeInternalCommand(int)));
		main->setItemParameter(id,KVI_INTERNALCOMMAND_DOCKWIDGET_SHOW);
	}

	main->insertSeparator();

	main->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_QUITAPP)),__tr2qs("&Quit"),g_pApp,SLOT(quit()));
}


void KviMenuBar::setupRecentServersPopup()
{
	g_pApp->fillRecentServersPopup((QPopupMenu *)sender());
}

void KviMenuBar::newConnectionToServer(int id)
{
	KviStr text = m_pRecentServersPopup->text(id);
	if(text.hasData())
	{
		KviStr szCommand;
		if(parseIrcUrl(text.ptr(),szCommand,KVI_IRCURL_CONTEXT_NEW))
		{
			g_pUserParser->parseCommandBuffer(szCommand.ptr(),m_pFrm->firstConsole());
		}
	}
}

void KviMenuBar::setupToolbarsPopup()
{
	m_pToolbarsPopup->clear();

	KviModuleExtensionDescriptorList * l = g_pModuleExtensionManager->getExtensionList("toolbar");
	if(l)
	{
		for(KviModuleExtensionDescriptor * d = l->first();d;d = l->next())
		{
			int id = m_pToolbarsPopup->insertItem(d->visibleName().ptr());
			m_pToolbarsPopup->setItemChecked(id,(m_pFrm->moduleExtensionToolBar(d->id())));
			m_pToolbarsPopup->setItemParameter(id,d->id());
		}
	}

#ifdef COMPILE_SCRIPTTOOLBAR
	QAsciiDictIterator<KviScriptToolBarDefinition> it(*(g_pScriptToolBarManager->toolBarDefinitions()));
	if(it.current())
	{
		m_pToolbarsPopup->insertSeparator();
		while(KviScriptToolBarDefinition * def = it.current())
		{
			int id = m_pToolbarsPopup->insertItem(def->label().ptr());
			m_pToolbarsPopup->setItemChecked(id,def->existingInstance(m_pFrm));
			m_pToolbarsPopup->setItemParameter(id,def->id());
			++it;
		}
	}
#endif // COMPILE_SCRIPTTOOLBAR
}

void KviMenuBar::toolbarsPopupSelected(int id)
{
	int idext = m_pToolbarsPopup->itemParameter(id);

#ifdef COMPILE_SCRIPTTOOLBAR
	KviScriptToolBarDefinition * def = g_pScriptToolBarManager->find(idext);
	if(def)
	{
		if(KviScriptToolBar * pBar = def->existingInstance(m_pFrm))
		{
			delete pBar; // kill it
		} else {
			def->instance(m_pFrm);
		}
		return;
	}
#endif // COMPILE_SCRIPTTOOLBAR

	if(KviModuleExtensionToolBar * t = m_pFrm->moduleExtensionToolBar(idext))
	{
		t->die();
	} else {
		g_pModuleExtensionManager->allocateExtension("toolbar",idext,m_pFrm);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Script items
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int KviMenuBar::getDefaultItemRealIndex(int iDefaultIndex)
{
	if(iDefaultIndex < 0)iDefaultIndex = 0;
	if(iDefaultIndex >= m_iNumDefaultItems)
		return indexOf(m_pDefaultItemId[m_iNumDefaultItems - 1]) + 1;
	return indexOf(m_pDefaultItemId[iDefaultIndex]);
}

KviScriptMenuBarItem * KviMenuBar::findMenu(const char * text)
{
	if(!m_pScriptItemList)return 0;
	for(KviScriptMenuBarItem * i = m_pScriptItemList->first();i;i = m_pScriptItemList->next())
	{
		if(kvi_strEqualCI(i->szText.ptr(),text))return i;
	}
	return 0;
}

KviScriptMenuBarItem * KviMenuBar::findMenu(KviPopupMenu * p)
{
	if(!m_pScriptItemList)return 0;
	for(KviScriptMenuBarItem * i = m_pScriptItemList->first();i;i = m_pScriptItemList->next())
	{
		if(i->pPopup == p)return i;
	}
	return 0;
}

bool KviMenuBar::removeMenu(const char * text)
{
	KviScriptMenuBarItem * i = findMenu(text);
	if(i)
	{
		disconnect(i->pPopup,SIGNAL(destroyed()),this,SLOT(menuDestroyed()));
		removeItem(i->id);
		m_pScriptItemList->removeRef(i);
		return true;
	}
	return false;
}

void KviMenuBar::menuDestroyed()
{
	KviScriptMenuBarItem * i = findMenu(((KviPopupMenu *)sender()));
	if(i)
	{
		removeItem(i->id);
		m_pScriptItemList->removeRef(i);
	}
}

void KviMenuBar::addMenu(const char * text,KviPopupMenu * p,int index)
{
	if(!m_pScriptItemList)
	{
		m_pScriptItemList = new KviPtrList<KviScriptMenuBarItem>;
		m_pScriptItemList->setAutoDelete(true);
	}/* else {
		removeMenu(text);
	}*/
	KviScriptMenuBarItem * it = new KviScriptMenuBarItem;
	it->szText = text;
	it->szPopupName = p->name();
	it->pPopup = p;
	it->id = insertItem(text,p,-1,index);
	connect(p,SIGNAL(destroyed()),this,SLOT(menuDestroyed()));
	m_pScriptItemList->append(it);
}

#include "kvi_menubar.moc"
