//
//   File : channelsjoinwindow.cpp
//   Creation date : Thu Nov 6 2001 12:41:18 by Juanjo lvarez
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2001 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the linkss of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "channelsjoinwindow.h"

#include "kvi_locale.h"
#include "kvi_options.h"
#include "kvi_selectors.h"
#include "kvi_app.h"
#include "kvi_uparser.h"
#include "kvi_string.h"
#include "kvi_iconmanager.h"
#include "kvi_console.h"

#include <qhbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

extern KviChannelsJoinWindow * g_pChannelsWindow;
extern QRect                   g_rectChannelsJoinGeometry;

KviChannelsJoinWindow::KviChannelsJoinWindow(QWidget * par, const char * name) 
: QWidget(par,name,Qt::WStyle_Customize | Qt::WStyle_Dialog)
{
	setCaption(__c2q(__tr("Join a channel")));

	setIcon(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CHANNEL)));

	QGridLayout * g = new QGridLayout(this,8,3,4,4);
	
	QLabel * l = new QLabel(__c2q(__tr("<center>Select a channel from the list or insert a channel name</center>")),this);
	g->addMultiCellWidget(l,0,0,0,1);


	m_pListView = new QListView(this);
	m_pListView->addColumn(__c2q(__tr("Channel")));
	m_pListView->setRootIsDecorated(true);
	m_pListView->setSelectionMode(QListView::Single);
	g->addMultiCellWidget(m_pListView,1,1,0,1);
	connect(m_pListView,SIGNAL(clicked(QListViewItem *)),this,SLOT(itemClicked(QListViewItem *)));
	connect(m_pListView,SIGNAL(doubleClicked(QListViewItem *)),this,SLOT(itemDoubleClicked(QListViewItem *)));

	l = new QLabel(__c2q(__tr("Channel:")),this);
	g->addWidget(l,2,0);

	m_pChannelEdit = new QLineEdit(this); 
	g->addWidget(m_pChannelEdit,2,1);
	connect(m_pChannelEdit,SIGNAL(returnPressed()), this, SLOT(editReturnPressed()));
	connect(m_pChannelEdit,SIGNAL(textChanged(const QString &)),this,SLOT(editTextChanged(const QString &)));


	l = new QLabel(__c2q(__tr("Password:")), this);
	g->addWidget(l,3,0);

	m_pPass = new QLineEdit(this);
	m_pPass->setEchoMode(QLineEdit::Password);
	g->addWidget(m_pPass,3,1);


	m_pJoinButton = new QPushButton(__c2q(__tr("Join")),this);
	connect(m_pJoinButton,SIGNAL(clicked()),this,SLOT(joinClicked()));
	m_pJoinButton->setEnabled(false);

	g->addMultiCellWidget(m_pJoinButton,4,4,0,1);


	m_pShowAtStartupCheck = new QCheckBox(__c2q(__tr("Show this window after connection to a server")),this);
	m_pShowAtStartupCheck->setChecked(KVI_OPTION_BOOL(KviOption_boolShowChannelsJoinOnIrc));
	g->addMultiCellWidget(m_pShowAtStartupCheck,5,5,0,1);


	QPushButton * cancelButton = new QPushButton(__c2q(__tr("Close")),this);
	cancelButton->setDefault(true);
	connect(cancelButton,SIGNAL(clicked()),this,SLOT(cancelClicked()));

	g->addMultiCellWidget(cancelButton,6,6,0,1);

/*
	QHBox * hb = new QHBox(this);
	hb->setSpacing(4);
	
	QPushButton * namesButton = new QPushButton("/names", hb);
	connect(namesButton,SIGNAL(clicked()),this,SLOT(namesClicked()));
	QPushButton * whoButton = new QPushButton("/who", hb);
	connect(whoButton,SIGNAL(clicked()),this,SLOT(whoClicked()));

	g->addWidget(hb,2,0);
*/

	g->setRowStretch(1,1);
	g->setColStretch(1,1);

	fillListView();

	if(g_rectChannelsJoinGeometry.y() < 5)g_rectChannelsJoinGeometry.setY(5);

	resize(g_rectChannelsJoinGeometry.width(),g_rectChannelsJoinGeometry.height());
	move(g_rectChannelsJoinGeometry.x(),g_rectChannelsJoinGeometry.y());
}

KviChannelsJoinWindow::~KviChannelsJoinWindow()
{
	KVI_OPTION_BOOL(KviOption_boolShowChannelsJoinOnIrc) = m_pShowAtStartupCheck->isChecked();

	g_rectChannelsJoinGeometry = QRect(pos().x(),pos().y(),size().width(),size().height());
	g_pChannelsWindow = 0;
}

void KviChannelsJoinWindow::fillListView()
{
	m_pListView->clear();

	QListViewItem * par = new QListViewItem(m_pListView,__tr("Recent channels"));
	par->setOpen(true);
	QListViewItem * chld;

	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it)
	{
		QListViewItem * chld = new QListViewItem(par,*it);
		chld->setPixmap(0,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CHANNEL)));
	}

	// FIXME: Registered channels go here!



}

void KviChannelsJoinWindow::itemClicked(QListViewItem * it) 
{
	if(!it)return;
	if(!it->parent())return;
	KviStr tmp = it->text(0);
	m_pChannelEdit->setText(tmp.ptr());
	enableJoin();
}

void KviChannelsJoinWindow::itemDoubleClicked(QListViewItem * it) 
{
	if(!it)return;
	if(!it->parent())return;
	KviStr tmp = it->text(0);
	m_pChannelEdit->setText(tmp.ptr());
	enableJoin();
	joinClicked();
}


void KviChannelsJoinWindow::editTextChanged(const QString &)
{
	enableJoin();
}

void KviChannelsJoinWindow::enableJoin()
{
	KviStr tmp = m_pChannelEdit->text();
	KviConsole * c = g_pApp->topmostConnectedConsole();
	if(tmp.isEmpty())
	{
		m_pJoinButton->setText(__tr("Join"));
		m_pJoinButton->setEnabled(false);
	} else {
		if(c)
		{
			m_pJoinButton->setText(__tr("Join"));
			m_pJoinButton->setEnabled(true);
		} else {
			m_pJoinButton->setText(__tr("You're not connected to a server"));
			m_pJoinButton->setEnabled(false);
		}
	}
}

void KviChannelsJoinWindow::cancelClicked()
{
	delete this;
}

void KviChannelsJoinWindow::joinClicked()
{
	KviStr pass = m_pPass->text();
	KviStr tmp = m_pChannelEdit->text();

	if(tmp.isEmpty())return;

	KviStr command(KviStr::Format,"join %s %s",tmp.ptr(),pass.ptr());

	KviConsole * c = g_pApp->topmostConnectedConsole();
	if(!c)return; // no connection
	KviWindow * w = g_pApp->activeWindow();
	if(w->console() != c)w = c;
    g_pUserParser->parseCommandBuffer(command.ptr(),w);

	m_pChannelEdit->setText("");
	m_pPass->setText("");
}

/*
void KviChannelsJoinWindow::whoClicked()
{
	KviStr tmp = m_pChannelEdit->text();
	if(!tmp.isEmpty())doCmd("who", tmp.ptr());
}

void KviChannelsJoinWindow::namesClicked()
{
	KviStr tmp = m_pChannelEdit->text();
	//FIXME: I must be a nice guy and implement /names in the core...
	if(!tmp.isEmpty())doCmd("raw names", tmp.ptr());
}
/*
void KviChannelsJoinWindow::itemDoubleClicked(QListBoxItem * it)
{
	if (it == 0)return;
	KviStr tmp = it->text();
	doCmd("join", tmp.ptr());
//	if(KVI_OPTION_BOOL(KviOption_boolCloseChannelsJoinAfterJoin))
//		g_pApp->collectGarbage(this);
}
*/

void KviChannelsJoinWindow::editReturnPressed()
{
	joinClicked();
}	

void KviChannelsJoinWindow::closeEvent(QCloseEvent *e)
{
	e->ignore();
	delete this;
}

#include "m_channelsjoinwindow.moc"
