//
//   File : libkvimircimport.cpp
//   Creation date : Tue Aug 27 01:20:35 2002 GMT by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2002 Szymon Stefanek (pragma at kvirc dot net)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "libkvimircimport.h"
#include "kvi_module.h"
#include "kvi_uparser.h"
#include "kvi_filedialog.h"
#include "kvi_locale.h"
#include "kvi_config.h"
#include "kvi_app.h"


#include <qmessagebox.h>
#include <qdir.h>

KviMircServersIniImportFilter * g_pMircServersIniImportFilter = 0;
KviRemoteMircServersIniImportFilter * g_pRemoteMircServersIniImportFilter = 0;


KviMircServersIniImportFilter::KviMircServersIniImportFilter()
: KviIrcServerImportFilter()
{
}

KviMircServersIniImportFilter::~KviMircServersIniImportFilter()
{
	g_pMircServersIniImportFilter = 0;
}

void KviMircServersIniImportFilter::doImport(const char * filename)
{
	KviConfig cfg(filename);
	if(cfg.hasGroup("servers"))
	{
		cfg.setGroup("servers");
		int i = 0;
		KviStr key;
		KviStr entry;
		do {
			key.sprintf("n%d",i);
			entry = cfg.readEntry(key.ptr(),"");
			if(entry.hasData())
			{
				KviStr description;
				KviStr serv;
				KviStr port;
				unsigned short int uPort;
				// <description>SERVER:<server:port>GROUP:<network>
				int idx = entry.findFirstIdx("SERVER:");
				if(idx != -1)
				{
					description = entry.left(idx);
					entry.cutLeft(idx + 7);
					idx = entry.findFirstIdx("GROUP:");
					if(idx != -1)
					{
						port = entry.left(idx);
						entry.cutLeft(idx + 6);
					}
					idx = port.findFirstIdx(':');
					if(idx != -1)
					{
						serv = port.left(idx);
						port.cutLeft(idx + 1);
						bool bOk;
						uPort = port.toUInt(&bOk);
						if(!bOk)uPort = 6667;
					} else {
						serv = port;
						uPort = 6667;
					}
				}
				if(entry.isEmpty())entry = __tr("orphan_servers");
				if(serv.hasData())
				{
					KviIrcServer s;
					s.m_szHostname = serv;
					s.m_szDescription = description;
					s.m_uPort = uPort;

					emit server(s,entry.ptr());
				}
				++i;
			}
		} while(entry.hasData());
	} else {
		KviStr tmp(KviStr::Format,__tr("%s doesn't look like a servers.ini file..."),filename);
		QMessageBox::warning(0,__c2q(__tr("KVIrc : Warning")),__c2q(tmp.ptr()));
	}
}

void KviMircServersIniImportFilter::start()
{
	KviStr buffer;
	if(!KviFileDialog::askForOpenFileName(buffer,__tr("Choose a servers.ini file"),0,"*.ini"))return;

	doImport(buffer.ptr());
	delete this;
}

void KviMircServersIniImportFilter::die()
{
	delete this;
}

KviRemoteMircServersIniImportFilter::KviRemoteMircServersIniImportFilter(const char * szUrl)
: KviMircServersIniImportFilter()
{
	m_szUrl = szUrl;
	m_pRequest = 0;
}

KviRemoteMircServersIniImportFilter::~KviRemoteMircServersIniImportFilter()
{
	if(m_pRequest)delete m_pRequest;
	g_pRemoteMircServersIniImportFilter = 0;
}

void KviRemoteMircServersIniImportFilter::start()
{
	if(m_pRequest)delete m_pRequest;

	m_pRequest = new KviHttpRequest();
	connect(m_pRequest,SIGNAL(terminated(bool)),this,SLOT(requestTerminated(bool)));

	g_pApp->getTmpFileName(m_szTmpFileName,"servers.ini");

	if(!m_pRequest->get(m_szUrl.ptr(),KviHttpRequest::StoreToFile,m_szTmpFileName.ptr()))
	{
		delete m_pRequest;
		m_pRequest = 0;
		return;
	}
}

void KviRemoteMircServersIniImportFilter::die()
{
	delete this;
}

void KviRemoteMircServersIniImportFilter::requestTerminated(bool bSuccess)
{
	if(!m_pRequest)return;
	delete m_pRequest;
	m_pRequest = 0;

	if(bSuccess)doImport(m_szTmpFileName.ptr());

	QDir d;
	d.remove(m_szTmpFileName.ptr());

	delete this;
}


static bool mircimport_module_init(KviModule * m)
{
	m->registerServerImportFilter(__tr("Import from servers.ini"));
	m->registerServerImportFilter(__tr("Import from www.mirc.co.uk"));

	return true;
}

static bool mircimport_module_cleanup(KviModule *m)
{
	if(g_pMircServersIniImportFilter)delete g_pMircServersIniImportFilter;
	if(g_pRemoteMircServersIniImportFilter)delete g_pRemoteMircServersIniImportFilter;

	m->unregisterMetaObject("KviMircServersIniImportFilter");
	m->unregisterMetaObject("KviRemoteMircServersIniImportFilter");

	return true;
}

static bool mircimport_module_can_unload(KviModule * m)
{
	return ((!g_pMircServersIniImportFilter) && (!g_pRemoteMircServersIniImportFilter));
}

KVIMODULEEXPORTFUNC KviIrcServerImportFilter * mircimport_module_createIrcServerImportFilter(const char *filterName)
{
	if(kvi_strEqualCI(filterName,__tr("Import from servers.ini")))
	{
		if(!g_pMircServersIniImportFilter)g_pMircServersIniImportFilter = new KviMircServersIniImportFilter();
		return g_pMircServersIniImportFilter;
	} else {
		if(!g_pRemoteMircServersIniImportFilter)g_pRemoteMircServersIniImportFilter = new KviRemoteMircServersIniImportFilter("http://www.mirc.co.uk/servers.ini");
		return g_pRemoteMircServersIniImportFilter;
	}
	return 0;
}

KVIMODULEEXPORTDATA KviModuleInfo kvirc_module_info=
{
	"File",                                                 // module name
	"1.0.0",                                                // module version
	"Copyright (C) 2002 Szymon Stefanek (pragma at kvirc dot net)", // author & (C)
	"Mirc servers.ini importer",
	mircimport_module_init,
	mircimport_module_can_unload,
	0,
	mircimport_module_cleanup
};

#include "libkvimircimport.moc"
