//
//   File : container.cpp
//   Creation date : Wed Nov 21 17:09:49 2001 GMT by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2001 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "container.h"
#include "instances.h"

#include "kvi_locale.h"
#include "kvi_iconmanager.h"
#include "kvi_app.h"

#include <qlayout.h>
#include <qpushbutton.h>

extern KviOptionsInstanceManager * g_pOptionsInstanceManager;

KviOptionsWidgetContainer::KviOptionsWidgetContainer(QWidget * par)
: QWidget(par,"container",Qt::WStyle_Customize | Qt::WStyle_Dialog)
{
	m_pOptionsWidget = 0;
}

KviOptionsWidgetContainer::~KviOptionsWidgetContainer()
{
	if(m_pOptionsWidget)delete m_pOptionsWidget;
}

void KviOptionsWidgetContainer::setup(KviOptionsWidget * w)
{
	QGridLayout * g = new QGridLayout(this,2,3,4,8);

	g->addMultiCellWidget(w,0,0,0,2);


	m_pCancel = new QPushButton(__c2q(__tr("Cancel")),this);
	g->addWidget(m_pCancel,1,1);
	m_pCancel->setDefault(true);
	connect(m_pCancel,SIGNAL(clicked()),this,SLOT(cancelClicked()));


	QPushButton * b = new QPushButton(__tr("OK"),this);
	b->setMinimumWidth(m_pCancel->sizeHint().width());
	g->addWidget(b,1,2);
	connect(b,SIGNAL(clicked()),this,SLOT(okClicked()));

	g->setRowStretch(0,1);
	g->setColStretch(0,1);

	KviOptionsWidgetInstanceEntry * e = g_pOptionsInstanceManager->findInstanceEntry(w->className());
	if(e)
	{
		setIcon(*(g_pIconManager->getSmallIcon(e->iIcon)));
		setCaption(e->szName);
	}
	m_pOptionsWidget = w;

}

void KviOptionsWidgetContainer::closeEvent(QCloseEvent *e)
{
	e->ignore();
	cancelClicked();
}

void KviOptionsWidgetContainer::showEvent(QShowEvent *e)
{
	if(parent() == 0)
	{
		move((g_pApp->desktop()->width() - width()) / 2,
			(g_pApp->desktop()->height() - height()) / 2);
	}
	QWidget::showEvent(e);
	m_pCancel->setFocus();
}


void KviOptionsWidgetContainer::okClicked()
{
	if(m_pOptionsWidget)m_pOptionsWidget->commit();
	delete this;
}

void KviOptionsWidgetContainer::cancelClicked()
{
	delete this;
}

#include "m_container.moc"
