//
//   File : optw_identity.cpp
//   Creation date : Tue Nov 14 2000 23:06:53 CEST by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "optw_identity.h"

#include <qlayout.h>
#include <qpushbutton.h>

#include "kvi_options.h"
#include "kvi_locale.h"
#include "kvi_uparser.h" 
#include "kvi_app.h"
#include "kvi_window.h"
#include "kvi_console.h"
#include "kvi_optionswidget.h"

//#warning "Info tips"

KviIdentityOptionsWidget::KviIdentityOptionsWidget(QWidget * parent)
: KviOptionsWidget(parent,"identity_options_widget")
{
    m_sModeStr = KVI_OPTION_STRING(KviOption_stringDefaultUserMode);    
    m_bI = m_sModeStr.contains('i',false);
    m_bW = m_sModeStr.contains('w',false);
    m_bS = m_sModeStr.contains('s',false);   
    
	createLayout(12,1); 
	addStringSelector(0,0,0,0,__tr("Nickname 1"),KviOption_stringNickname1);
	addStringSelector(0,1,0,1,__tr("Nickname 2"),KviOption_stringNickname2);
	addStringSelector(0,2,0,2,__tr("Nickname 3"),KviOption_stringNickname3);
	addStringSelector(0,3,0,3,__tr("Nickname 4"),KviOption_stringNickname4);
	addStringSelector(0,4,0,4,__tr("Username"),KviOption_stringUsername);
	addStringSelector(0,5,0,5,__tr("Real name"),KviOption_stringRealname);
    addLabel(0,6,0,6,__tr("Default user mode:"));
    m_pISelector = addBoolSelector(0,7,0,7,__tr("Invisible"),&m_bI);
    m_pWSelector = addBoolSelector(0,8,0,8,__tr("Wallops"),&m_bW);
    m_pSSelector = addBoolSelector(0,9,0,9,__tr("Server Notices"),&m_bS);
    
	addSeparator(0,10,2,10);
    
	addPixmapSelector(0,11,2,11,__tr("Avatar image"),KviOption_pixmapMyAvatar);
    
	
	bool b_Iscon = g_pApp->activeWindow()->console()->isConnected();

	// It MUST be a better way to do this...
	m_pChangeButton1 = new QPushButton(__tr("Change now"),this);
	m_pChangeButton2 = new QPushButton(__tr("Change now"),this);
	m_pChangeButton3 = new QPushButton(__tr("Change now"),this);
	m_pChangeButton4 = new QPushButton(__tr("Change now"),this);

	layout()->addWidget(m_pChangeButton1,0,2);
	layout()->addWidget(m_pChangeButton2,1,2);
	layout()->addWidget(m_pChangeButton3,2,2);
	layout()->addWidget(m_pChangeButton4,3,2);

	m_pChangeButton1->setEnabled(b_Iscon);
	m_pChangeButton2->setEnabled(b_Iscon);
	m_pChangeButton3->setEnabled(b_Iscon);
	m_pChangeButton4->setEnabled(b_Iscon);

	connect(m_pChangeButton1,SIGNAL(clicked()),this,SLOT(but1Clicked()));
	connect(m_pChangeButton2,SIGNAL(clicked()),this,SLOT(but2Clicked()));
	connect(m_pChangeButton3,SIGNAL(clicked()),this,SLOT(but3Clicked()));
	connect(m_pChangeButton4,SIGNAL(clicked()),this,SLOT(but4Clicked()));
    
    connect(m_pISelector,SIGNAL(toggled(bool)),this,SLOT(iModeCheckToggled(bool)));
    connect(m_pWSelector,SIGNAL(toggled(bool)),this,SLOT(wModeCheckToggled(bool)));
    connect(m_pSSelector,SIGNAL(toggled(bool)),this,SLOT(sModeCheckToggled(bool)));
	
	layout()->setRowStretch(11,1);
}

KviIdentityOptionsWidget::~KviIdentityOptionsWidget()
{
}

void KviIdentityOptionsWidget::but1Clicked(){changeNick(1);}
void KviIdentityOptionsWidget::but2Clicked(){changeNick(2);}
void KviIdentityOptionsWidget::but3Clicked(){changeNick(3);}
void KviIdentityOptionsWidget::but4Clicked(){changeNick(4);}

void KviIdentityOptionsWidget::iModeCheckToggled(bool){changeStatus('i');}
void KviIdentityOptionsWidget::wModeCheckToggled(bool){changeStatus('w');}
void KviIdentityOptionsWidget::sModeCheckToggled(bool){changeStatus('s');}

void KviIdentityOptionsWidget::changeStatus(char c)
{    
    KviStr tmp(m_sModeStr);
    KviStr tmp2(m_sModeStr);
    
    bool iToggled = m_pISelector->isChecked();
    bool wToggled = m_pWSelector->isChecked();
    bool sToggled = m_pSSelector->isChecked();
    bool actualCheck;
    
    switch(c)
    {
        case 'i': actualCheck = iToggled;break;
        case 'w': actualCheck = wToggled;break;
        case 's': actualCheck = sToggled;break;
    }
    
    if(actualCheck && !m_sModeStr.contains(c))m_sModeStr.append(c);
    else if(m_sModeStr.contains(c))
    { // Remove the char let's hope there is no more than ONE ocurrence of it (it SHOULDN'T)    
        m_sModeStr = tmp.cutToFirst(c);   
        m_sModeStr.append(tmp2.cutFromFirst(c));    
    }     
}        

void KviIdentityOptionsWidget::butModeClicked(void){}

void KviIdentityOptionsWidget::changeNick(short i)
{
	KviStr command;
	KviStr nick;
	switch(i)
	{
		case 1:
			nick = KVI_OPTION_STRING(KviOption_stringNickname1);
			break;
		case 2:
			nick = KVI_OPTION_STRING(KviOption_stringNickname2);
			break;
		case 3:
			nick = KVI_OPTION_STRING(KviOption_stringNickname3);
			break;
		case 4:
			nick = KVI_OPTION_STRING(KviOption_stringNickname4);
			break;
	}
	command.sprintf("nick %s",nick.ptr());
	g_pUserParser->parseCommandBuffer(command.ptr(),g_pApp->activeWindow());
}

void KviIdentityOptionsWidget::commit(void)
{
    KviOptionsWidget::commit();
    KVI_OPTION_STRING(KviOption_stringDefaultUserMode) = m_sModeStr;
}
#include "m_optw_identity.moc"
