//
//   File : libkvitb_options.cpp
//   Creation date : Sun Oct 21 2001 13:29:12 CEST by Szymon Stefanek
//
//   Copyright (C) 2001 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "libkvitb_options.h"

#include "kvi_module.h"
#include "kvi_uparser.h"

#include "kvi_frame.h"
#include "kvi_window.h"
#include "kvi_locale.h"
#include "kvi_app.h"
#include "kvi_imagelib.h"
#include "kvi_mdimanager.h"

#include <qtoolbutton.h>

static KviPtrList<KviOptionsToolBar> * g_pToolBarList = 0;

KviOptionsToolBar::KviOptionsToolBar(KviModuleExtensionDescriptor * d,KviFrame * par)
: KviModuleExtensionToolBar(d,__tr("Options"),par)
{
	g_pToolBarList->append(this);

	KviStr buffer;

	g_pApp->findImage(buffer,"tb_options.png");

	KviImageLibrary lib(buffer.ptr(),32,32);

	QToolButton * tb;
	tb = new QToolButton(lib.getImage(0),__tr("Servers"),__tr("Server options"),this,SLOT(showServerOptions()),this);
	tb = new QToolButton(lib.getImage(1),__tr("Identity"),__tr("Identity options"),this,SLOT(showIdentityOptions()),this);
	tb = new QToolButton(lib.getImage(2),__tr("Global settings"),__tr("Global settings"),this,SLOT(showGlobalOptions()),this);

}

KviOptionsToolBar::~KviOptionsToolBar()
{
	g_pToolBarList->removeRef(this);
}


void KviOptionsToolBar::die()
{
//	debug("DYING (OPTIONS)");
	delete this;
}

void KviOptionsToolBar::showServerOptions()
{
	g_pUserParser->parseCommandBuffer("options.edit KviServerOptionsWidget",g_pApp->activeWindow());
}

void KviOptionsToolBar::showIdentityOptions()
{
	g_pUserParser->parseCommandBuffer("options.edit KviIdentityOptionsWidget",g_pApp->activeWindow());
}

void KviOptionsToolBar::showGlobalOptions()
{
	g_pUserParser->parseCommandBuffer("options.dialog",g_pApp->activeWindow());
}


static KviModuleExtension * tb_options_alloc(KviModuleExtensionDescriptor *d,void *f)
{
	return new KviOptionsToolBar(d,(KviFrame *)f);
}


static bool tb_options_module_init(KviModule * m)
{
	g_pToolBarList = new KviPtrList<KviOptionsToolBar>;
	g_pToolBarList->setAutoDelete(false);

	m->registerExtension("toolbar","options",__tr("Options"),tb_options_alloc);

	return true;
}

static bool tb_options_module_cleanup(KviModule *m)
{
	while(g_pToolBarList->first())delete g_pToolBarList->first();
	delete g_pToolBarList;

	m->unregisterMetaObject("KviOptionsToolBar");

	return true;
}

static bool tb_options_module_can_unload(KviModule *m)
{
	return (g_pToolBarList->isEmpty());
}

KVIMODULEEXPORTDATA KviModuleInfo kvirc_module_info=
{
	"List",                                                      // module name
	"1.0.0",                                                     // module version
	"Copyright (C) 2000-2001 Szymon Stefanek (stefanek@tin.it)", // author & (C)
	"Options toolbar extension",
	tb_options_module_init,
	tb_options_module_can_unload,
	0,
	tb_options_module_cleanup
};

#include "libkvitb_options.moc"
